#include <windows.h>
#include "tagWrapper.h"

#include "myapeTag.h"
#include "myflacTag.h"
#include "dummyTag.h"

static const TAG_READER_INTERFACE *readers[] = {&apeIF, &flacIF, &dummyIF, };

typedef struct{
	void* tag;
	BYTE* CUEBody; // pointer to cuesheet body
	DWORD CUELength;
	BYTE* pPos; // pointer to current position(in CUEBody)
	const TAG_READER_INTERFACE* reader;
	CueCharset charset;
} CUE_OBJECT;


PCUE_OBJECT CueOpen(HANDLE hFile){
	int i,j; // loop counter.
	int FieldNum;
	int IFlen = sizeof(readers)/sizeof(readers[0]);
	void* tag; // tag object.
	BYTE* value;
	DWORD valSize;
	const TAG_READER_INTERFACE* reader; // pointer to active reader.

	CUE_OBJECT* cueobj = NULL;

	for(i=0; i<IFlen; i++){
		reader = readers[i];
		tag = (reader->OpenTag(hFile));
		if(tag == NULL)continue;

		FieldNum = reader->GetFieldNumber(tag);
		for(j=0; j<FieldNum; j++){
			if(_stricmp( reader->GetFieldName(tag,j),"cuesheet") == 0){
				value = reader->GetFieldValue(tag,j,&valSize);

				// when success finding cuesheet field.
				cueobj = (CUE_OBJECT*)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(CUE_OBJECT));
				if(cueobj == NULL)return NULL;
				cueobj->tag = tag;
				cueobj->reader = reader;
				cueobj->charset = reader->GetFieldCharset(tag,j);
				cueobj->CUEBody = value;
				cueobj->pPos = value;
				cueobj->CUELength = valSize;
				return cueobj;
			}
		}
	}
	return NULL;
}

//
// CueGetTag(cue,1,"artist") -> track 1artist̒l擾
// APE TagVorbis CommentUTF8Ȃ̂ŁAUTF8ŕԂĂ܂
// ̕NULLI[łȂ̂ŁAsizeKǂނ
BYTE* CueGetTag(PCUE_OBJECT cueobj,int index,const char* field,LPDWORD size){
	CUE_OBJECT* cueobject = (CUE_OBJECT*)cueobj;
	char fieldName[128] = "";   // NOTE: obt@TCYŒ̂߁Afield̒ɐ!!
	int FieldNum;
	int i;
	wsprintfA(fieldName,"Cue_track%02d_",index);
	strcat_s(fieldName,sizeof(fieldName),field);
	FieldNum = cueobject->reader->GetFieldNumber(cueobject->tag);
	for(i=0; i<FieldNum; i++){
		if(_stricmp( cueobject->reader->GetFieldName(cueobject->tag,i),fieldName) == 0){
			return cueobject->reader->GetFieldValue(cueobject->tag,i,size);
		}
	}
	return NULL;
}

CueCharset CueGetCharset(PCUE_OBJECT cueobj){
	CUE_OBJECT* cueobject = (CUE_OBJECT*)cueobj;
	return cueobject->charset;
}

BOOL CueReadFile(PCUE_OBJECT cueobj, void* buffer, size_t size, LPDWORD read){
	CUE_OBJECT* cueobject = (CUE_OBJECT*)cueobj;
	DWORD availSize;
	DWORD copySize;

	availSize = cueobject->CUEBody + cueobject->CUELength - cueobject->pPos;
	copySize = min(size,availSize);
	memcpy(buffer,cueobject->pPos,copySize);
	cueobject->pPos += copySize;
	if(read){*read = copySize;}
	return TRUE;
}

BOOL CueSeekFile(PCUE_OBJECT cueobj, size_t pos){
	CUE_OBJECT* cueobject = (CUE_OBJECT*)cueobj;
	if(pos < cueobject->CUELength){
		cueobject->pPos = cueobject->CUEBody + pos;
		return TRUE;
	}
	return FALSE;
}

void CueClose(PCUE_OBJECT cueobj){
	CUE_OBJECT* cueobject = (CUE_OBJECT*)cueobj;
	cueobject->reader->CloseTag(cueobject->tag);
	HeapFree(GetProcessHeap(),0,cueobject);
}

